@@ -44,6 +44,8 @@ void Image::Initialize(Handle<Object> target) {
     constructor->InstanceTemplate()->SetInternalFieldCount(1);
     constructor->SetClassName(String::NewSymbol("Image"));
 
+    NODE_SET_PROTOTYPE_METHOD(constructor, "getPixel", getPixel);
+    NODE_SET_PROTOTYPE_METHOD(constructor, "setPixel", setPixel);
     NODE_SET_PROTOTYPE_METHOD(constructor, "encodeSync", encodeSync);
     NODE_SET_PROTOTYPE_METHOD(constructor, "encode", encode);
     NODE_SET_PROTOTYPE_METHOD(constructor, "view", view);
@@ -176,6 +178,62 @@ void Image::set_prop(Local<String> property,
     }
 }
 
+Handle<Value> Image::getPixel(const Arguments& args)
+{
+    HandleScope scope;
+    int x = 0;
+    int y = 0;
+    if (args.Length() >= 2) {
+        if (!args[0]->IsNumber())
+            return ThrowException(Exception::TypeError(
+                                      String::New("first arg, 'x' must be an integer")));
+        if (!args[1]->IsNumber())
+            return ThrowException(Exception::TypeError(
+                                      String::New("second arg, 'y' must be an integer")));
+        x = args[0]->IntegerValue();
+        y = args[1]->IntegerValue();
+    } else {
+        return ThrowException(Exception::TypeError(
+                                  String::New("must supply x,y to query pixel color")));
+    }
+    Image* im = node::ObjectWrap::Unwrap<Image>(args.This());
+    mapnik::image_data_32 const& data = im->this_->data();
+    if (x >= 0 && x < static_cast<int>(data.width())
+        && y >= 0 && y < static_cast<int>(data.height()))
+    {
+        unsigned pixel = data(x,y);
+        unsigned r = pixel & 0xff;
+        unsigned g = (pixel >> 8) & 0xff;
+        unsigned b = (pixel >> 16) & 0xff;
+        unsigned a = (pixel >> 24) & 0xff;
+        return Color::New(mapnik::color(r,g,b,a));
+    }
+    return Undefined();
+}
+
+Handle<Value> Image::setPixel(const Arguments& args)
+{
+    HandleScope scope;
+    if (args.Length() < 3 || (!args[0]->IsNumber() && !args[1]->IsNumber())) {
+        return ThrowException(Exception::TypeError(String::New("expects three arguments: x, y, and pixel value")));
+    }
+    Local<Object> obj = args[2]->ToObject();
+    if (obj->IsNull() || obj->IsUndefined() || !Color::constructor->HasInstance(obj))
+        return ThrowException(Exception::TypeError(String::New("mapnik.Color expected as third arg")));
+    Color * color = node::ObjectWrap::Unwrap<Color>(obj);
+    int x = args[0]->IntegerValue();
+    int y = args[1]->IntegerValue();
+    Image* im = node::ObjectWrap::Unwrap<Image>(args.This());
+    mapnik::image_data_32 & data = im->this_->data();
+    if (x < static_cast<int>(data.width()) && y < static_cast<int>(data.height()))
+    {
+        data(x,y) = color->get()->rgba();
+        return scope.Close(Undefined());
+    }
+    return ThrowException(Exception::TypeError(String::New("invalid pixel requested")));
+    return scope.Close(Undefined());
+}
+
 Handle<Value> Image::clearSync(const Arguments& args)
 {
     HandleScope scope;
@@ -508,7 +566,16 @@ typedef struct {
     bool error;
     std::string error_name;
     Persistent<Function> cb;
-} image_ptr_baton_t;
+} image_mem_ptr_baton_t;
+
+typedef struct {
+    uv_work_t request;
+    image_ptr im;
+    std::string filename;
+    bool error;
+    std::string error_name;
+    Persistent<Function> cb;
+} image_file_ptr_baton_t;
 
 Handle<Value> Image::open(const Arguments& args)
 {
@@ -534,11 +601,9 @@ Handle<Value> Image::open(const Arguments& args)
         return ThrowException(Exception::TypeError(
                                   String::New("last argument must be a callback function")));
 
-    image_ptr_baton_t *closure = new image_ptr_baton_t();
+    image_file_ptr_baton_t *closure = new image_file_ptr_baton_t();
     closure->request.data = closure;
-    std::string filename = TOSTR(args[0]);
-    closure->data = filename.data();
-    closure->dataLength = filename.size();
+    closure->filename = TOSTR(args[0]);
     closure->error = false;
     closure->cb = Persistent<Function>::New(Handle<Function>::Cast(callback));
     uv_queue_work(uv_default_loop(), &closure->request, EIO_Open, (uv_after_work_cb)EIO_AfterOpen);
@@ -547,20 +612,19 @@ Handle<Value> Image::open(const Arguments& args)
 
 void Image::EIO_Open(uv_work_t* req)
 {
-    image_ptr_baton_t *closure = static_cast<image_ptr_baton_t *>(req->data);
+    image_file_ptr_baton_t *closure = static_cast<image_file_ptr_baton_t *>(req->data);
 
     try
     {
-        std::string filename(closure->data,closure->dataLength);
-        boost::optional<std::string> type = mapnik::type_from_filename(filename);
+        boost::optional<std::string> type = mapnik::type_from_filename(closure->filename);
         if (!type)
         {
             closure->error = true;
-            closure->error_name = "Unsupported image format: " + filename;
+            closure->error_name = "Unsupported image format: " + closure->filename;
         }
         else
         {
-            std::auto_ptr<mapnik::image_reader> reader(mapnik::get_image_reader(filename,*type));
+            std::auto_ptr<mapnik::image_reader> reader(mapnik::get_image_reader(closure->filename,*type));
             if (reader.get())
             {
                 closure->im = boost::make_shared<mapnik::image_32>(reader->width(),reader->height());
@@ -569,7 +633,7 @@ void Image::EIO_Open(uv_work_t* req)
             else
             {
                 closure->error = true;
-                closure->error_name = "Failed to load: " + filename;
+                closure->error_name = "Failed to load: " + closure->filename;
             }
         }
     }
@@ -583,7 +647,7 @@ void Image::EIO_Open(uv_work_t* req)
 void Image::EIO_AfterOpen(uv_work_t* req)
 {
     HandleScope scope;
-    image_ptr_baton_t *closure = static_cast<image_ptr_baton_t *>(req->data);
+    image_file_ptr_baton_t *closure = static_cast<image_file_ptr_baton_t *>(req->data);
     TryCatch try_catch;
     if (closure->error || !closure->im)
     {
@@ -631,8 +695,7 @@ Handle<Value> Image::fromBytesSync(const Arguments& args)
             reader->read(0,0,image_ptr->data());
             Image* im = new Image(image_ptr);
             Handle<Value> ext = External::New(im);
-            Handle<Object> obj = constructor->GetFunction()->NewInstance(1, &ext);
-            return scope.Close(obj);
+            return scope.Close(constructor->GetFunction()->NewInstance(1, &ext));
         }
         return ThrowException(Exception::TypeError(String::New(
                                                        "Failed to load from buffer")));
@@ -675,19 +738,19 @@ Handle<Value> Image::fromBytes(const Arguments& args)
         return ThrowException(Exception::TypeError(
                                   String::New("last argument must be a callback function")));
 
-    image_ptr_baton_t *closure = new image_ptr_baton_t();
+    image_mem_ptr_baton_t *closure = new image_mem_ptr_baton_t();
     closure->request.data = closure;
     closure->data = node::Buffer::Data(obj);
     closure->dataLength = node::Buffer::Length(obj);
     closure->error = false;
     closure->cb = Persistent<Function>::New(Handle<Function>::Cast(callback));
-    uv_queue_work(uv_default_loop(), &closure->request, EIO_FromBytes, (uv_after_work_cb)EIO_AfterOpen);
+    uv_queue_work(uv_default_loop(), &closure->request, EIO_FromBytes, (uv_after_work_cb)EIO_AfterFromBytes);
     return Undefined();
 }
 
 void Image::EIO_FromBytes(uv_work_t* req)
 {
-    image_ptr_baton_t *closure = static_cast<image_ptr_baton_t *>(req->data);
+    image_mem_ptr_baton_t *closure = static_cast<image_mem_ptr_baton_t *>(req->data);
 
     try
     {
@@ -710,6 +773,30 @@ void Image::EIO_FromBytes(uv_work_t* req)
     }
 }
 
+void Image::EIO_AfterFromBytes(uv_work_t* req)
+{
+    HandleScope scope;
+    image_mem_ptr_baton_t *closure = static_cast<image_mem_ptr_baton_t *>(req->data);
+    TryCatch try_catch;
+    if (closure->error || !closure->im)
+    {
+        Local<Value> argv[1] = { Exception::Error(String::New(closure->error_name.c_str())) };
+        closure->cb->Call(Context::GetCurrent()->Global(), 1, argv);
+    }
+    else
+    {
+        Image* im = new Image(closure->im);
+        Handle<Value> ext = External::New(im);
+        Local<Object> image_obj = constructor->GetFunction()->NewInstance(1, &ext);
+        Local<Value> argv[2] = { Local<Value>::New(Null()), Local<Value>::New(ObjectWrap::Unwrap<Image>(image_obj)->handle_) };
+        closure->cb->Call(Context::GetCurrent()->Global(), 2, argv);
+    }
+    if (try_catch.HasCaught()) {
+        node::FatalException(try_catch);
+    }
+    closure->cb.Dispose();
+    delete closure;
+}
 
 Handle<Value> Image::encodeSync(const Arguments& args)
 {
