@@ -6,6 +6,7 @@
 #include "mapnik_map.hpp"
 #include "mapnik_image.hpp"
 #include "mapnik_grid.hpp"
+#include "mapnik_feature.hpp"
 #include "mapnik_cairo_surface.hpp"
 #ifdef SVG_RENDERER
 #include <mapnik/svg/output/svg_renderer.hpp>
@@ -21,6 +22,7 @@
 
 #include <mapnik/map.hpp>
 #include <mapnik/layer.hpp>
+#include <mapnik/geom_util.hpp>
 #include <mapnik/version.hpp>
 #include <mapnik/request.hpp>
 #include <mapnik/feature.hpp>
@@ -50,6 +52,81 @@
 #include <vector>                       // for vector
 
 
+template <typename PathType>
+bool _hit_test(PathType & path, double x, double y, double tol)
+{
+    double x0 = 0;
+    double y0 = 0;
+    path.rewind(0);
+    mapnik::eGeomType geom_type = static_cast<mapnik::eGeomType>(path.type());
+    switch(geom_type)
+    {
+    case mapnik::Point:
+    {
+        unsigned command = path.vertex(&x0, &y0);
+        if (command == mapnik::SEG_END) return false;
+        return mapnik::distance(x, y, x0, y0) <= tol;
+        break;
+    }
+    case mapnik::Polygon:
+    {
+        double x1 = 0;
+        double y1 = 0;
+        bool inside = false;
+        unsigned command = path.vertex(&x0, &y0);
+        if (command == mapnik::SEG_END) return false;
+        while (mapnik::SEG_END != (command = path.vertex(&x1, &y1)))
+        {
+            if (command == mapnik::SEG_CLOSE) continue;
+            if (command == mapnik::SEG_MOVETO)
+            {
+                x0 = x1;
+                y0 = y1;
+                continue;
+            }
+            if ((((y1 <= y) && (y < y0)) ||
+                 ((y0 <= y) && (y < y1))) &&
+                (x < (x0 - x1) * (y - y1)/ (y0 - y1) + x1))
+            {
+                inside=!inside;
+            }
+            x0 = x1;
+            y0 = y1;
+        }
+        return inside;
+        break;
+    }
+    case mapnik::LineString:
+    {
+        double x1 = 0;
+        double y1 = 0;
+        unsigned command = path.vertex(&x0, &y0);
+        if (command == mapnik::SEG_END) return false;
+        while (mapnik::SEG_END != (command = path.vertex(&x1, &y1)))
+        {
+            if (command == mapnik::SEG_CLOSE) continue;
+            if (command == mapnik::SEG_MOVETO)
+            {
+                x0 = x1;
+                y0 = y1;
+                continue;
+            }
+            double distance = mapnik::point_to_segment_distance(x,y,x0,y0,x1,y1);
+            if (distance < tol)
+                return true;
+            x0 = x1;
+            y0 = y1;
+        }
+        return false;
+        break;
+    }
+    default:
+        return false;
+        break;
+    }
+    return false;
+}
+
 Persistent<FunctionTemplate> VectorTile::constructor;
 
 void VectorTile::Initialize(Handle<Object> target) {
@@ -61,6 +138,7 @@ void VectorTile::Initialize(Handle<Object> target) {
     NODE_SET_PROTOTYPE_METHOD(constructor, "setData", setData);
     NODE_SET_PROTOTYPE_METHOD(constructor, "setDataSync", setDataSync);
     NODE_SET_PROTOTYPE_METHOD(constructor, "getData", getData);
+    NODE_SET_PROTOTYPE_METHOD(constructor, "query", query);
     NODE_SET_PROTOTYPE_METHOD(constructor, "names", names);
     NODE_SET_PROTOTYPE_METHOD(constructor, "toJSON", toJSON);
     NODE_SET_PROTOTYPE_METHOD(constructor, "toGeoJSON", toGeoJSON);
@@ -164,6 +242,141 @@ Handle<Value> VectorTile::painted(const Arguments& args)
     return scope.Close(Boolean::New(d->painted()));
 }
 
+Handle<Value> VectorTile::query(const Arguments& args)
+{
+    HandleScope scope;
+    if (args.Length() < 2 || !args[0]->IsNumber() || !args[1]->IsNumber())
+    {
+        return ThrowException(Exception::Error(
+                                  String::New("expects lon,lat args")));
+    }
+    double tolerance = 0.0; // meters
+    std::string layer_name("");
+    if (args.Length() > 2)
+    {
+        Local<Object> options = Object::New();
+        if (!args[2]->IsObject())
+        {
+            return ThrowException(Exception::TypeError(String::New("optional third argument must be an options object")));
+        }
+        options = args[2]->ToObject();
+        if (options->Has(String::NewSymbol("tolerance")))
+        {
+            Local<Value> tol = options->Get(String::New("tolerance"));
+            if (!tol->IsNumber())
+            {
+                return ThrowException(Exception::TypeError(String::New("tolerance value must be a number")));
+            }
+            tolerance = tol->NumberValue();
+        }
+        if (options->Has(String::NewSymbol("layer")))
+        {
+            Local<Value> layer_id = options->Get(String::New("layer"));
+            if (!layer_id->IsString())
+            {
+                return ThrowException(Exception::TypeError(String::New("layer value must be a string")));
+            }
+            layer_name = TOSTR(layer_id);
+        }
+    }
+
+    double lon = args[0]->NumberValue();
+    double lat = args[1]->NumberValue();
+    mapnik::projection wgs84("+init=epsg:4326");
+    mapnik::projection merc("+init=epsg:3857");
+    mapnik::proj_transform tr(wgs84,merc);
+    double x = lon;
+    double y = lat;
+    double z = 0;
+    if (!tr.forward(x,y,z))
+    {
+        return ThrowException(Exception::Error(
+                                  String::New("could not reproject lon/lat to mercator")));
+    }
+    VectorTile* d = node::ObjectWrap::Unwrap<VectorTile>(args.This());
+    mapnik::vector::tile const& tiledata = d->get_tile();
+    Local<Array> arr = Array::New();
+    mapnik::coord2d pt(x,y);
+    unsigned idx = 0;
+    if (!layer_name.empty())
+    {
+            int tile_layer_idx = -1;
+            for (int j=0; j < tiledata.layers_size(); ++j)
+            {
+                mapnik::vector::tile_layer const& layer = tiledata.layers(j);
+                if (layer_name == layer.name())
+                {
+                    tile_layer_idx = j;
+                    break;
+                }
+            }
+            if (tile_layer_idx > -1)
+            {
+                mapnik::vector::tile_layer const& layer = tiledata.layers(tile_layer_idx);
+                boost::shared_ptr<mapnik::vector::tile_datasource> ds = boost::make_shared<
+                                            mapnik::vector::tile_datasource>(
+                                                layer,
+                                                d->x_,
+                                                d->y_,
+                                                d->z_,
+                                                d->width()
+                                                );
+                mapnik::featureset_ptr fs = ds->features_at_point(pt,tolerance);
+                if (fs)
+                {
+                    mapnik::feature_ptr feature;
+                    while ((feature = fs->next()))
+                    {
+                        bool hit = false;
+                        BOOST_FOREACH ( mapnik::geometry_type const& geom, feature->paths() )
+                        {
+                           if (_hit_test(geom,x,y,tolerance))
+                           {
+                               hit = true;
+                               break;
+                           }
+                        }
+                        if (hit) arr->Set(idx++,Feature::New(feature));
+                    }
+                }
+            }
+    }
+    else
+    {
+        for (int i=0; i < tiledata.layers_size(); ++i)
+        {
+            mapnik::vector::tile_layer const& layer = tiledata.layers(i);
+            boost::shared_ptr<mapnik::vector::tile_datasource> ds = boost::make_shared<
+                                        mapnik::vector::tile_datasource>(
+                                            layer,
+                                            d->x_,
+                                            d->y_,
+                                            d->z_,
+                                            d->width()
+                                            );
+            mapnik::featureset_ptr fs = ds->features_at_point(pt,tolerance);
+            if (fs)
+            {
+                mapnik::feature_ptr feature;
+                while ((feature = fs->next()))
+                {
+                    bool hit = false;
+                    BOOST_FOREACH ( mapnik::geometry_type const& geom, feature->paths() )
+                    {
+                       if (_hit_test(geom,x,y,tolerance))
+                       {
+                           hit = true;
+                           break;
+                       }
+                    }
+                    if (hit) arr->Set(idx++,Feature::New(feature));
+                }
+            }
+        }
+    }
+    return scope.Close(arr);
+}
+
 Handle<Value> VectorTile::toJSON(const Arguments& args)
 {
     HandleScope scope;
@@ -271,17 +484,17 @@ static void layer_to_geojson(mapnik::vector::tile_layer const& layer,
         Local<String> js_type = String::New("Unknown");
         switch (g_type)
         {
-        case mapnik::datasource::Point:
+        case mapnik::Point:
         {
             js_type = String::New("Point");
             break;
         }
-        case mapnik::datasource::LineString:
+        case mapnik::LineString:
         {
             js_type = String::New("LineString");
             break;
         }
-        case mapnik::datasource::Polygon:
+        case mapnik::Polygon:
         {
             js_type = String::New("Polygon");
             break;
@@ -293,7 +506,7 @@ static void layer_to_geojson(mapnik::vector::tile_layer const& layer,
         }
         geometry->Set(String::NewSymbol("type"),js_type);
         Local<Array> g_arr = Array::New();
-        if (g_type == mapnik::datasource::Polygon)
+        if (g_type == mapnik::Polygon)
         {
             Local<Array> enclosing_array = Array::New(1);
             enclosing_array->Set(0,g_arr);
@@ -330,7 +543,7 @@ static void layer_to_geojson(mapnik::vector::tile_layer const& layer,
                     double y2 = y1;
                     if (tr.forward(x2,y2,zc))
                     {
-                        if (g_type == mapnik::datasource::Point)
+                        if (g_type == mapnik::Point)
                         {
                             g_arr->Set(0,Number::New(x2));
                             g_arr->Set(1,Number::New(y2));
